import base64
import marshal
import os
import py_compile
import tempfile

from django.conf.urls import url
from django.shortcuts import render
from django.utils.safestring import mark_safe
from toga.interface.app import App as AppInterface
from toga.interface.widgets.base import Widget

from .bootstrap import bootstrap
from .window import Window


class MainWindow(Window):
    def on_close(self, widget, data):
        pass


class App(AppInterface):
    _MAIN_WINDOW_CLASS = MainWindow


    def __init__(self, interface):
        self.interface = interface
        self.interface._impl = self
        self.container = None
        self.create()

    def create(self):
        self.startup()

    def main_loop(self):
        pass

    def open_document(self, fileURL):
        self.interface.factory.not_implemented('App.open_document()')

    def create_menus(self):
        self.interface.factory.not_implemented('App.create_menus()')

    def exit(self):
        pass

    def set_on_exit(self, value):
        pass

    def current_window(self):
        self.interface.factory.not_implemented('App.current_window()')

    def enter_full_screen(self, windows):
        self.interface.factory.not_implemented('App.enter_full_screen()')

    def exit_full_screen(self, windows):
        self.interface.factory.not_implemented('App.exit_full_screen()')

    def show_cursor(self):
        self.interface.factory.not_implemented('App.show_cursor()')

    def hide_cursor(self):
        self.interface.factory.not_implemented('App.hide_cursor()')

    # ====

    # def materialize(self):
    #     app = render.App(self.name, self.app_id, self.ports)
    #     app.main_window = self.main_window.materialize()
    #     for win_id, win in self.windows:
    #         app.windows.append(win.materialize())
    #     return app

    def __str__(self):
        return mark_safe(self.main_window._impl.__html__())

    def get_urls(self):
        urlpatterns = [
            url(r'^$', self.home, name='home'),
        ] + self.main_window.get_urls()
        for win_id, window in self.windows:
            urlpatterns += window.get_urls()
        return urlpatterns

    @property
    def urls(self):
        return self.get_urls(), 'toga', self.name

    @property
    def ports(self):
        return ",".join(
            "%s=%s" % (name, widget.id)
            for name, widget in self.__dict__.items()
            if isinstance(widget, Widget)
        )

    def home(self, request):
        # app = self.app.materialize()
        # if app.main_window.id == self.id:
        #     window = app.main_window
        # else:
        #     try:
        #         window = app.windows[self.id]
        #     except KeyError:
        #         raise Exception("Unknown window")
        sourcefile = os.path.join(os.path.dirname(__file__), 'impl', '__init__.py')

        fd, tempname = tempfile.mkstemp()
        py_compile.compile(sourcefile, cfile=tempname, doraise=True)
        with open(os.path.join(os.path.dirname(sourcefile), tempname), 'rb') as compiled:
            toga = base64.encodebytes(compiled.read())

        widgets = {}
        for widget in ["app", "window", "box", "button", "label", "textinput", "webview"]:
            sourcefile = os.path.join(os.path.dirname(__file__), 'impl', "%s.py" % widget)

            fd, tempname = tempfile.mkstemp()
            py_compile.compile(sourcefile, cfile=tempname, doraise=True)
            with open(os.path.join(os.path.dirname(sourcefile), tempname), 'rb') as compiled:
                bytecode = base64.encodebytes(compiled.read())
                widgets['toga.%s' % widget] = {
                    'filename': sourcefile,
                    'bytecode': bytecode,
                }

        context = {
            'toga': toga,
            'widgets': widgets,
            'bootstrap': base64.encodebytes(b'\xee\x0c\r\n00000000' + marshal.dumps(bootstrap.__code__)).strip(),
            'app': self,
            'callbacks': {
                # 'sample': base64.encodebytes(b'\x08\x1c\xe8VU\x00\x00\x00' + marshal.dumps(sample.__code__)).strip()
                '%s-%s' % (widget, message): {
                    'filename': '<string>',
                    'bytecode': base64.encodebytes(b'\xee\x0c\r\n00000000' + marshal.dumps(callback.__code__)).strip()
                }
                for (widget, message), callback in self.main_window.callbacks.items()
            }
        }
        return render(request, 'toga/app.html', context)



from .base import Widget
from .. import impl


class Label(Widget):
    def create(self):
        self._impl = impl.Label(
            id=self.id,
            text=self._config['text'],
            alignment=self._config['alignment'],
            style=self.style,
        )

    def set_text(self, value):
        self._impl.innerText = self.interface._text

    def set_alignment(self, value):
        self.interface.factory.not_implemented('Label.set_alignment()')

    def rehint(self):
        pass






class Window:
    def __init__(self, id, title, content=None):
        self.id = id

        self.title = title
        self.set_content(content)

    def __html__(self):
        return """
            <nav id="toga:%s" data-toga-class="toga.Window" data-toga-ports="%s" class="navbar navbar-fixed-top navbar-dark bg-inverse">
                <a class="navbar-brand" href="#">%s</a>
                <ul class="nav navbar-nav">
                    <!--li class="nav-item active">
                      <a class="nav-link" href="#">Home <span class="sr-only">(current)</span></a>
                    </li-->
                </ul>
            </nav>""" % (
                self.id,
                '',  #  self.ports,
                self.title
            ) + self.content.__html__()

    def set_title(self, title):
        self.title = title
        dom.window.title = title

    def set_content(self, content):
        self.content = content
        if content:
            self.content.parent = self

    # HTML popups don't allow for titles, so we'll prepend it
    def info_dialog(self, title, message):
        dom.alert("%s\n\n%s" % (title, message));





class Button:
    def __init__(self, id, label, ports=None, on_press=None, style=None):
        self.id = id
        self._impl = None
        self.label = label

        self.ports = ports if ports else {}

        self.on_press = on_press
        self.style = style

    def __html__(self):
        return '<button id="toga:%s" class="toga Button btn btn-default" style="%s" data-toga-class="toga.Button" data-toga-parent="%s" data-toga-ports="%s" data-toga-on-press="%s">%s</button>' % (
            self.id,
            self.style,
            self.parent.id,
            '',  #  self.ports,
            self.on_press,
            self.label
        )



from .. import impl


class Button(Widget):
    def create(self):
        self._impl = impl.Button(
            id=self.id,
            label=self._config['label'],
            on_press=self.handler(self._config['on_press'], 'on_press') if self._config['on_press'] else None,
            style=self.style,
        )

    def set_window(self, window):
        super()._set_window(window)
        if self.on_press:
            self.window.callbacks[(self.id, 'on_press')] = self.on_press

    def set_label(self, label):
        pass

    def set_enabled(self, value):
        pass

    def set_background_color(self, value):
        pass

    def set_on_press(self, handler):
        pass

    def rehint(self):
        pass




def bootstrap(element):
    import toga
    parts = element.dataset.togaClass.split('.')
    bootstrap_method = getattr(toga, 'bootstrap_' + parts[1])
    result = bootstrap_method(element)
    return result



import base64
import marshal
import os
import py_compile

from django.shortcuts import render
from django.utils.safestring import mark_safe
from toga.interface.window import Window as WindowInterface

from . import dialogs
from . import impl
from .bootstrap import bootstrap
from .container import Container


class Window(WindowInterface):
    _IMPL_CLASS = impl.Window
    _CONTAINER_CLASS = Container
    _DIALOG_MODULE = dialogs

    def __init__(self, interface):
        self.interface = interface
        self.interface._impl = self
        self.container = None
        self.create()

    def create(self):
        self._impl = impl.Window(
            id=self.id,
            title=self._config['title']
        )

    def set_toolbar(self, items):
        pass

    def set_content(self, widget):
        self._impl.set_content(widget._impl)

    def set_title(self, title):
        # self._impl.set_title(title)
        pass

    def set_app(self, app):
        pass

    def set_position(self, position):
        pass

    def set_size(self, size):
        pass

    def on_close(self, widget, data):
        pass

    def create_toolbar(self, toolbar):
        pass

    def __str__(self):
        return mark_safe(self._impl.__html__())

    def get_urls(self):
        # def wrap(view, cacheable=False):
        #     def wrapper(*args, **kwargs):
        #         return self.admin_view(view, cacheable)(*args, **kwargs)
        #     wrapper.admin_site = self
        #     return update_wrapper(wrapper, view)

        # Admin-site-wide views.
        urlpatterns = [
        ]
        return urlpatterns

    def show(self):
        pass

    def set_full_screen(self, is_full_screen):
        self.interface.factory.not_implemented('Window.set_full_screen()')


    def home(self, request):
        # app = self.app.materialize()
        # if app.main_window.id == self.id:
        #     window = app.main_window
        # else:
        #     try:
        #         window = app.windows[self.id]
        #     except KeyError:
        #         raise Exception("Unknown window")

        sourcefile = os.path.join(os.path.dirname(__file__), 'render', '__init__.py')

        fd, tempname = tempfile.mkstemp()
        py_compile.compile(sourcefile, cfile=tempname, doraise=True)
        with open(os.path.join(os.path.dirname(sourcefile), tempname), 'rb') as compiled:
            toga = base64.encodebytes(compiled.read())

        return render(request, 'toga/app.html', {
            'toga': toga,
            'bootstrap': base64.encodebytes(b'\xee\x0c\r\n00000000' + marshal.dumps(bootstrap.__code__)).strip(),
            'window': self._impl,
            'callbacks': {
                # 'sample': base64.encodebytes(b'\x08\x1c\xe8VU\x00\x00\x00' + marshal.dumps(sample.__code__)).strip()
                '%s-%s' % (widget, message): base64.encodebytes(b'\xee\x0c\r\n00000000' + marshal.dumps(callback.__code__)).strip()
                for (widget, message), callback in self.callbacks.items()
            }
        })

    def info_dialog(self, title, message):
        self.interface.factory.not_implemented('Window.info_dialog()')

    def question_dialog(self, title, message):
        self.interface.factory.not_implemented('Window.question_dialog()')

    def confirm_dialog(self, title, message):
        self.interface.factory.not_implemented('Window.confirm_dialog()')

    def error_dialog(self, title, message):
        self.interface.factory.not_implemented('Window.error_dialog()')

    def stack_trace_dialog(self, title, message, content, retry=False):
        self.interface.factory.not_implemented('Window.stack_trace_dialog()')

    def save_file_dialog(self, title, suggested_filename, file_types):
        self.interface.factory.not_implemented('Window.save_file_dialog()')



from toga.interface import TextInput as TextInputInterface

from .base import WidgetMixin
from .. import impl


class TextInput(TextInputInterface, WidgetMixin):
    def __init__(self, id=None, initial='', placeholder=None, readonly=False, style=None):
        super().__init__(id=id, style=style, initial=initial, placeholder=placeholder, readonly=readonly)
        self._create()

    def create(self):
        self._impl = impl.TextInput(
            id=self.id,
            initial=self._config['initial'],
            placeholder=self._config['placeholder'],
            readonly=self._config['readonly'],
            # on_press=self.handler(self._config['on_press'], 'on_press') if self._config['on_press'] else None
            style=self.style,
        )

    # def _set_window(self, window):
    #     super()._set_window(window)
    #     if self.on_press:
    #         self.window.callbacks[(self.id, 'on_press')] = self.on_press

    def set_placeholder(self, value):
        self.interface.factory.not_implemented('TextInput.set_placeholder()')

    def set_readonly(self, value):
        self.interface.factory.not_implemented('TextInput.set_readonly()')

    def get_value(self):
        return self._impl.value

    def set_value(self, value):
        self.interface.factory.not_implemented('TextInput.set_value()')

    def set_alignment(self, value):
        pass

    def set_font(self, value):
        pass

    def rehint(self):
        pass

    def set_on_change(self, handler):
        pass



from toga.interface import WebView as WebViewInterface

from .base import WidgetMixin
from .. import impl


class WebView(WebViewInterface, WidgetMixin):
    def __init__(self, id=None, url=None, on_key_down=None, style=None):
        super().__init__(id=id, style=style, url=url, on_key_down=on_key_down)
        self._create()

    def create(self):
        self._impl = impl.WebView(
            id=self.id,
            url=self._config['url'],
            on_key_down=self.handler(self._config['on_key_down'], 'on_key_down') if self._config['on_key_down'] else None,
            style=self.style,
        )

    def set_url(self, value):
        self._impl.url = value

    # def _set_window(self, window):
    #     super()._set_window(window)
    #     if self.on_press:
    #         self.window.callbacks[(self.id, 'on_press')] = self.on_press

    def get_dom(self):
        raise NotImplementeException()

    def set_user_agent(self, value):
        raise NotImplementeException()

    def set_placeholder(self, value):
        raise NotImplementeException()

    def set_readonly(self, value):
        raise NotImplementeException()

    def get_value(self):
        return self._impl.value

    def set_value(self, value):
        raise NotImplementeException()

    def set_content(self, root_url, content):
        raise NotImplementeException()

    def evaluate(self, javascript):
        raise NotImplementeException()
